package come.stormborntales.notevault.ui.viewmodel

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import come.stormborntales.notevault.data.local.entity.NoteCollection
import come.stormborntales.notevault.data.local.entity.NoteEntity
import come.stormborntales.notevault.data.repository.CollectionRepository
import come.stormborntales.notevault.data.repository.NoteRepository
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch

class NoteBrowserViewModel (
    private val noteRepository: NoteRepository,
    private val collectionRepository: CollectionRepository
) : ViewModel() {
    private val _currentCollections = MutableStateFlow<List<NoteCollection>>(emptyList())
    val currentCollections: StateFlow<List<NoteCollection>> = _currentCollections.asStateFlow()

    private val _currentNotes = MutableStateFlow<List<NoteEntity>>(emptyList())
    val currentNotes: StateFlow<List<NoteEntity>> = _currentNotes.asStateFlow()

    private val _currentParentId = MutableStateFlow<Int?>(null)
    val currentParentId: StateFlow<Int?> = _currentParentId.asStateFlow()

    private val _pathStack = MutableStateFlow<List<NoteCollection>>(emptyList())
    val pathStack: StateFlow<List<NoteCollection>> = _pathStack.asStateFlow()

    init {
        loadContentForParent(null)
    }

    fun loadContentForParent(parentId: Int?, selectedCollection: NoteCollection? = null) {
        _currentParentId.value = parentId
        viewModelScope.launch {
            launch {
                collectionRepository.getCollectionsByParent(parentId).collect {
                    _currentCollections.value = it
                }
            }
            launch {
                noteRepository.getNotesForCollection(parentId).collect {
                    _currentNotes.value = it
                }
            }
            if (selectedCollection != null) {
                _pathStack.value = _pathStack.value + selectedCollection
            } else if (parentId == null) {
                _pathStack.value = emptyList()
            }
        }
    }

    fun createCollection(name: String) {
        viewModelScope.launch {
            val newCollection = NoteCollection(name = name, parentId = _currentParentId.value)
            collectionRepository.insertCollection(newCollection)
            loadContentForParent(_currentParentId.value)
        }
    }

    fun navigateToLevel(index: Int) {
        val newStack = _pathStack.value.take(index + 1)
        _pathStack.value = newStack
        val newParent = newStack.lastOrNull()?.id
        loadContentForParent(newParent)
    }

    fun navigateUp() {
        val newStack = _pathStack.value.dropLast(1)
        _pathStack.value = newStack
        val newParentId = newStack.lastOrNull()?.id
        loadContentForParent(newParentId)
    }
}
